;----------------------------------------------------------------------------
; Sega Master System & Game Gear - PSG functions
; by Omar Cornut / Bock|Zoop (c) in February 2001
;----------------------------------------------------------------------------

; PSG_INIT() ----------------------------------------------------------------
; Initialize PSG
;----------------------------------------------------------------------------
; no parameters
;----------------------------------------------------------------------------
psg_init:
                ld      hl, psg_init_table      ; Mute PSG
                ld      b, 4                    ;
                ld      c, PORT_PSG             ;
                otir                            ;
                ret
psg_init_table:
.db             $9F, $BF, $DF, $FF
;----------------------------------------------------------------------------

; PSG_PLAY() ----------------------------------------------------------------
; Start song playing
;----------------------------------------------------------------------------
; hl = pointer to song
;----------------------------------------------------------------------------
psg_play:
                ld      (VAR_psg_song), hl
                ld      (VAR_psg_song_p), hl
                ld      a, 1
                ld      (VAR_psg_delay), a
                ret
;----------------------------------------------------------------------------

; PSG_STOP() ----------------------------------------------------------------
; Stop song
;----------------------------------------------------------------------------
; no parameters
;----------------------------------------------------------------------------
psg_stop:
                ld      hl, 0
                ld      (VAR_psg_song), hl
                ld      (VAR_psg_song_p), hl
                ld      a, 1
                ld      (VAR_psg_delay), a
                jp      psg_init
                ;ret
;----------------------------------------------------------------------------

; PSG_UPDATE() --------------------------------------------------------------
; Update currently played song. Must be called once per frame!
;----------------------------------------------------------------------------
; no parameters
;----------------------------------------------------------------------------
psg_update:
                ld      hl, (VAR_psg_song)              ; Check if a song
                ld      a, h                            ; is actually playing
                or      l                               ;
                jr      nz, psg_update_do               ;
                ret

psg_update_do:
                ld      a, (VAR_psg_delay)              ; Reduce delay
                dec     a                               ;
                jr      nz, psg_update_end              ;

                ld      hl, (VAR_psg_song_p)            ; Set song pointer
                ld      a, (hl)                         ; Read chunk size

                and     $FF                             ; If chunk size is 0
                jr      z, psg_update_song_end          ; -> end of song

                inc     hl                              ; Read data and write
                ld      b, a                            ; it back to PSG port
                ld      c, PORT_PSG                     ;
                otir                                    ;

                ld      a, (hl)                         ; Read psg_delay
                inc     hl
                ld      (VAR_psg_song_p), hl            ; Save song pointer

psg_update_end:
                ld      (VAR_psg_delay), a              ; Save delay
                ret

psg_update_song_end:
                ld      hl, (VAR_psg_song)              ; This is the end of
                ld      (VAR_psg_song_p), hl            ; song, so set delay to
                ld      a, 1                            ; 1 and restart song
                jr      psg_update_end
;----------------------------------------------------------------------------

